/************************************************************
 * Stack.h
 *  - sicherere Stack-Klasse
 ************************************************************/
#ifndef _STACK_H
#define _STACK_H

#include <deque>
#include <exception>

template <class T>
class Stack {
  protected:
    std::deque<T> c;        // Container fuer die Elemente

  public:
    /* Exception-Klasse fuer pop() bei leerem Stack
     */
    class ReadEmptyStack : public std::exception {
      public:
        virtual const char* what() const throw() {
            return "read empty stack";
        }
    };
  
    // Anzahl der Elemente
    std::deque<T>::size_type size() const {
        return c.size();
    }

    // leer ?
    bool empty() const {
        return c.empty();
    }

    // Zugriff auf oberstes Element
    T& top () {
        if (c.empty()) {
            throw ReadEmptyStack();
        }
        return c.back();
    }

    // Element einkellern
    void push (const T& elem) {
        c.push_back(elem);
    }

    // Element auskellern
    T pop () {
        if (c.empty()) {
            throw ReadEmptyStack();
        }
        T elem(c.back());
        c.pop_back();
        return elem;
    }
};

#endif /* _STACK_H */
